//
// Created by alex on 18-12-8.
//
//
#include <opencv2/opencv.hpp>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <string>
#include <iostream>
#include <zconf.h>
#include <csignal>
#include <thread>
#include "MvGmslCamera.h"
#include <fstream>
#include <chrono>

using namespace std::chrono;

using std::string;
sig_atomic_t exitRequested = 0;

void handler(int) {
    std::cout << "will exit..." << std::endl;
    exitRequested = true;
}

struct sample_context{
    uint32_t window_num{4};
    string cam_devname;
    string cam_fmt;
    unsigned int cam_pixfmt;
    unsigned int cam_w{1280};
    unsigned int cam_h{720};
    unsigned char camera_no;
    struct sync_out_a_cfg_client_t stCameraCfgSend{};
};

static void
print_usage(void) {
    printf("\n\tUsage: example [OPTIONS]\n\n"
           "\tExample: \n"
           "\t./cameras_opencv_demo -d /dev/video0 -s 1280x720\n\n"
           "\tSupported options:\n"
           "\t-d\t\tSet V4l2 video device node\n"
           "\t-s\t\tSet output resolution of video device\n"
           "\t-n\t\tSet sync and async camera no. for example: [-n 2-4] the forward one is sync cameras no 2,the after one is async cameras no 4 (8 sync cameras is setted by default.like [-n 8-0])\n"
           "\t-r\t\tSet sync and async camera freq for example: [-r 30-20] the forward one is sync cameras freq 30,the after one is async cameras freq 20(sync freq 30 is setted by default.like [-r 30-0]) \n"
           "\t-b\t\tSet which cameras you want to trigger.example: [-b 0x0f-0xf0] the forward one is sync cameras which you want trigger,the after one is async cameras which you want trigger(all 8 cameras is setted sync model by default.like[-b 0xff-0])\n"
           "\t-p\t\tSet async cameras is triggered at which angle in a circle,not set by default.\n"
           "\t-h\t\tPrint this usage\n\n"
           "\tNOTE: It runs infinitely until you terminate it with <ctrl+c>\n");
}


static bool
parse_cmdline(struct sample_context *ctx, int argc, char **argv) {
    int c;
    int sync_camera_bit_draw = 0,async_camera_bit_draw = 0;

    if (argc < 2) {
        print_usage();
        exit(EXIT_SUCCESS);
    }

    while ((c = getopt(argc, argv, "d:s:r:n:b:p:h")) != -1) {
        switch (c) {
            case 'd':
                ctx->cam_devname = optarg;
            sscanf(ctx->cam_devname.c_str(),"/dev/video%hhu",&ctx->camera_no);
                break;
            case 's':
                if (sscanf(optarg, "%ux%u",&ctx->cam_w, &ctx->cam_h) != 2) {
                    print_usage();
                    return false;
                }
                ctx->cam_fmt = optarg;
                break;
            case 'r':
                if (sscanf(optarg, "%hhu-%hhu",&ctx->stCameraCfgSend.sync_freq, &ctx->stCameraCfgSend.async_freq) != 2) {
                   print_usage();
                    return false;
                }
                           printf("sync_freq : %d async_freq:%d\n",ctx->stCameraCfgSend.sync_freq,ctx->stCameraCfgSend.async_freq);
                break;
            case 'n':
                if (sscanf(optarg, "%hhu-%hhu",&ctx->stCameraCfgSend.sync_camera_num, &ctx->stCameraCfgSend.async_camera_num) != 2) {
                   print_usage();
                    return false;
                }
                           printf("sync_camera_num : %d async_camera_num:%d\n",(ctx->stCameraCfgSend.sync_camera_num),(ctx->stCameraCfgSend.async_camera_num));
                break;
            case 'b':
                if (sscanf(optarg, "%x-%x",&sync_camera_bit_draw, &async_camera_bit_draw) != 2) {
                   print_usage();
                   return false;
                }
                   ctx->stCameraCfgSend.sync_camera_bit_draw = (unsigned char)sync_camera_bit_draw;
                   ctx->stCameraCfgSend.async_camera_bit_draw = (unsigned char)async_camera_bit_draw;
                   printf("sync_camera_bit_draw : %d async_camera_bit_draw:%d\n",ctx->stCameraCfgSend.sync_camera_bit_draw,ctx->stCameraCfgSend.async_camera_bit_draw);
                break;
           case 'p':
               if (sscanf(optarg, "%hhu-%hhu-%hhu-%hhu-%hhu-%hhu-%hhu-%hhu",
                          &ctx->stCameraCfgSend.async_camera_pos[0], &ctx->stCameraCfgSend.async_camera_pos[1],&ctx->stCameraCfgSend.async_camera_pos[2],
                          &ctx->stCameraCfgSend.async_camera_pos[3],&ctx->stCameraCfgSend.async_camera_pos[4],&ctx->stCameraCfgSend.async_camera_pos[5]
                          ,&ctx->stCameraCfgSend.async_camera_pos[6],&ctx->stCameraCfgSend.async_camera_pos[7]) != 8) {
                   print_usage();
                   return false;
               }
                  printf("pos:[0]:%hhu [1]:%hhu [2]:%hhu [3]:%hhu [4]:%hhu [5]:%hhu [6]:%hhu [7]:%hhu \n",ctx->stCameraCfgSend.async_camera_pos[0],ctx->stCameraCfgSend.async_camera_pos[1],ctx->stCameraCfgSend.async_camera_pos[2],
                  ctx->stCameraCfgSend.async_camera_pos[3],ctx->stCameraCfgSend.async_camera_pos[4],ctx->stCameraCfgSend.async_camera_pos[5],ctx->stCameraCfgSend.async_camera_pos[6],ctx->stCameraCfgSend.async_camera_pos[7]);
               break;
            case 'h':
                print_usage();
                exit(EXIT_SUCCESS);
                break;
            default:
                print_usage();
                return false;
        }
    }
    return true;
}

int main(int argc, char *argv[]) {
    struct sample_context ctx = {};

    struct sync_out_a_cfg_client_t stCameraCfgSend = {0};
    stCameraCfgSend.async_camera_num = 0;
    stCameraCfgSend.async_freq = 0;
    stCameraCfgSend.async_camera_bit_draw = 0;
    stCameraCfgSend.sync_camera_num = 8;
    stCameraCfgSend.sync_freq = 30;
    stCameraCfgSend.sync_camera_bit_draw = 0xff;
    ctx.cam_fmt = " ";
    ctx.cam_devname = "/dev/video0";
    ctx.cam_h = 720;
    ctx.cam_w = 1280;
    ctx.window_num = 1;
    ctx.stCameraCfgSend = stCameraCfgSend;

    if (!parse_cmdline(&ctx, argc, argv)) {
        return -1;
    }
    uint imgWidth{640};
    uint imgHeight{360};
    string imgFmt = {"ABGR32"};
//    string imgFmt = {"YUYV"};

    miivii::MvGmslCamera mvcam(ctx.cam_devname, ctx.window_num, ctx.cam_w, ctx.cam_h, ctx.stCameraCfgSend, imgWidth, imgHeight,
                               imgFmt);

    std::string windowName("DisplayCamera ");
    for (uint32_t i = 0; i < ctx.window_num; i++) {
        cv::namedWindow(windowName + std::to_string(ctx.camera_no), cv::WindowFlags::WINDOW_AUTOSIZE);
        cv::moveWindow(windowName + std::to_string(ctx.camera_no), 200 * ctx.camera_no, 200 * ctx.camera_no);
    }
    cv::Mat outMat[ctx.window_num];
    uint8_t *outbuf[ctx.window_num];
    cv::Mat imgbuf[ctx.window_num];
    signal(SIGINT, &handler);
    bool quit = false;
    uint64_t timestamp{};
    while (!quit) {
        if (exitRequested) {
            quit = true;
            break;
        }
        if (mvcam.GetImageCvMat(outMat, timestamp,ctx.camera_no)) {
            for (uint32_t i = 0; i < ctx.window_num; i++) {
                if (imgFmt == "YUYV") {
                    cv::cvtColor(outMat[i], outMat[i], cv::COLOR_YUV2BGR_YUYV);
                    cv::imshow(windowName + std::to_string(ctx.camera_no), outMat[i]);
                } else if (imgFmt == "ABGR32") {
                    cv::cvtColor(outMat[i], outMat[i], cv::COLOR_RGBA2BGR);
                    cv::imshow(windowName + std::to_string(ctx.camera_no), outMat[i]);
                }
            }
            std::cout << "Timestamp from GetImageCvMat:  " << timestamp << std::endl;
        } else {
            std::cerr << "Can't get image form camera." << std::endl;
        }
        if (cv::waitKey(1) == 27) {// Wait for 'esc' key press to exit
            break;
        }

//        }
        /*use raw data to get image*/

            if (mvcam.GetImagePtr(outbuf, timestamp,ctx.camera_no)) {
                for (uint32_t i = 0; i < ctx.window_num; i++) {
                    if (imgFmt == "YUYV") {
                        imgbuf[i] = cv::Mat(imgHeight, imgWidth, CV_8UC2, outbuf[i]);
                        cv::Mat mrgba(imgHeight, imgWidth, CV_8UC3);
                        cv::cvtColor(imgbuf[i], mrgba, cv::COLOR_YUV2BGR_YUYV);
                        cv::imshow(windowName + std::to_string(ctx.camera_no), mrgba);
                    } else if (imgFmt == "ABGR32") {
                        imgbuf[i] = cv::Mat(imgHeight, imgWidth, CV_8UC4, outbuf[i]);
                        cv::cvtColor(imgbuf[i], imgbuf[i], cv::COLOR_RGBA2BGR);
                        cv::imshow(windowName + std::to_string(ctx.camera_no), imgbuf[i]);
                    }
                }
                std::cout << "Timestamp from GetImagePtr  :  " << timestamp << std::endl;
            } else {
                std::cerr << "Can't get image form camera." << std::endl;
            }

        if (cv::waitKey(1) == 27) {// Wait for 'esc' key press to exit
            break;
        }

    }
    return 0;
}

